@@ -105,7 +105,7 @@ Another interesting feature is to convert ASTs to YAML:
 ```
 '''
 
-from typing import Any, Callable, cast, List, Mapping, Optional, Set, Tuple, Union
+from typing import Any, Callable, List, Mapping, Optional, Sequence, Set, Tuple, Union, cast
 from functools import singledispatch
 from copy import copy
 from re import fullmatch
@@ -116,44 +116,50 @@ from clingo import ast
 from clingo.ast import (
     AST, ASTSequence, ASTType, Function, Location, Position, StrSequence,
     SymbolicTerm, SymbolicAtom, TheoryAtomType, TheoryFunction,
-    TheoryOperatorType, Transformer)
+    TheoryOperatorType, Transformer, UnaryOperation)
 from .theory import is_operator
 
 __all__ = ['ast_to_dict', 'dict_to_ast', 'location_to_str',
            'prefix_symbolic_atoms', 'rename_symbolic_atoms', 'str_to_location',
            'theory_parser_from_definition', 'Arity', 'Associativity',
-           'AtomTable', 'OperatorTable', 'SymbolicAtomRenamer', 'TheoryParser',
-           'TheoryTermParser', 'TheoryUnparsedTermParser',
-           'TheoryUnparsedTermParser']
+           'AtomTable', 'OperatorTable', 'TheoryParser', 'TheoryTermParser',
+           'TheoryUnparsedTermParser', 'TheoryUnparsedTermParser']
 
 
 class Arity(Enum):
     '''
     Enumeration of operator arities.
     '''
+    # pylint:disable=invalid-name
     Unary = 1
     Binary = 2
 
+
 class Associativity(Enum):
     '''
     Enumeration of operator associativities.
     '''
+    # pylint: disable=invalid-name
     Left = auto()
     Right = auto()
     NoAssociativity = auto()
 
+
 def _s(m, a: str, b: str):
     '''
     Select the match group b if not None and group a otherwise.
     '''
     return m[a] if m[b] is None else m[b]
 
+
 def _quote(s: str) -> str:
     return s.replace('\\', '\\\\').replace(':', '\\:')
 
+
 def _unquote(s: str) -> str:
     return s.replace('\\:', ':').replace('\\\\', '\\')
 
+
 def location_to_str(loc: Location) -> str:
     """
     This function transfroms a loctation object into a readable string.
@@ -172,21 +178,22 @@ def location_to_str(loc: Location) -> str:
     """
     begin, end = loc.begin, loc.end
     bf, ef = _quote(begin.filename), _quote(end.filename)
-    ret = "{}:{}:{}".format(bf, begin.line, begin.column)
+    ret = f"{bf}:{begin.line}:{begin.column}"
     dash, eq = True, bf == ef
     if not eq:
-        ret += "{}{}".format("-" if dash else ":", ef)
+        ret += f"{'-' if dash else ':'}{ef}"
         dash = False
     eq = eq and begin.line == end.line
     if not eq:
-        ret += "{}{}".format("-" if dash else ":", end.line)
+        ret += f"{'-' if dash else ':'}{end.line}"
         dash = False
     eq = eq and begin.column == end.column
     if not eq:
-        ret += "{}{}".format("-" if dash else ":", end.column)
+        ret += f"{'-' if dash else ':'}{end.column}"
         dash = False
     return ret
 
+
 def str_to_location(loc: str) -> Location:
     """
     This function parses a location from its string representation.
@@ -213,13 +220,15 @@ def str_to_location(loc: str) -> Location:
     end = Position(_unquote(_s(m, 'bf', 'ef')), int(_s(m, 'bl', 'el')), int(_s(m, 'bc', 'ec')))
     return Location(begin, end)
 
+
 OperatorTable = Mapping[Tuple[str, Arity],
-                    Tuple[int, Associativity]]
+                        Tuple[int, Associativity]]
 AtomTable = Mapping[Tuple[str, int],
                     Tuple[TheoryAtomType,
                           str,
                           Optional[Tuple[List[str], str]]]]
 
+
 class TheoryUnparsedTermParser:
     """
     Parser for unparsed theory terms in clingo's AST that works like the
@@ -277,8 +286,8 @@ class TheoryUnparsedTermParser:
             self._terms.append(TheoryFunction(b.location, operator, [b]))
         else:
             a = self._terms.pop()
-            l = Location(a.location.begin, b.location.end)
-            self._terms.append(TheoryFunction(l, operator, [a, b]))
+            loc = Location(a.location.begin, b.location.end)
+            self._terms.append(TheoryFunction(loc, operator, [a, b]))
 
     def check_operator(self, operator: str, arity: Arity, location: Location) -> None:
         """
@@ -295,7 +304,7 @@ class TheoryUnparsedTermParser:
             Location of the operator for error reporting.
         """
         if (operator, arity) not in self._table:
-            raise RuntimeError("cannot parse operator `{}`: {}".format(operator, location_to_str(location)))
+            raise RuntimeError(f"cannot parse operator `{operator}`: {location_to_str(location)}")
 
     def parse(self, x: AST) -> AST:
         """
@@ -334,6 +343,7 @@ class TheoryUnparsedTermParser:
 
         return self._terms[0]
 
+
 class TheoryTermParser(Transformer):
     """
     Parser for theory terms in clingo's AST that works like the inbuilt one.
@@ -393,6 +403,7 @@ class TheoryTermParser(Transformer):
         """
         return cast(AST, self(self._parser.parse(x)))
 
+
 class TheoryParser(Transformer):
     """
     This class parses theory atoms in the same way as clingo's internal parser.
@@ -594,6 +605,7 @@ class TheoryParser(Transformer):
 
         return x
 
+
 def theory_parser_from_definition(x: AST) -> TheoryParser:
     """
     Turn an AST node of type TheoryDefinition into a TheoryParser.
@@ -641,27 +653,18 @@ def theory_parser_from_definition(x: AST) -> TheoryParser:
     return TheoryParser(terms, atoms)
 
 
-class SymbolicAtomRenamer(Transformer):
+class _SymbolicAtomTransformer(Transformer):
     '''
-    A transformer to rename symbolic atoms.
-
-    Parameters
-    ----------
-    rename_function
-        A function for renaming symbols.
-
-    See Also
-    --------
-    rename_symbolic_atoms
+    Transforms symbolic atoms with the given function.
     '''
     # pylint: disable=invalid-name
 
-    def __init__(self, rename_function: Callable[[str], str]):
-        self._rename_function = rename_function
+    def __init__(self, transformer_function: Callable[[AST], AST]):
+        self._transformer_function = transformer_function
 
     def visit_SymbolicAtom(self, x: AST) -> AST:
         '''
-        Rename the given symbolic atom and return the renamed version.
+        Transform the given symbolic.
 
         Parameters
         ----------
@@ -673,13 +676,28 @@ class SymbolicAtomRenamer(Transformer):
         The rewritten AST.
         '''
         term = x.symbol
-        if term.ast_type == ASTType.SymbolicTerm:
-            sym = term.symbol
-            term = SymbolicTerm(term.location,
-                                clingo.Function(self._rename_function(sym.name), sym.arguments, sym.positive))
-        elif term.ast_type == ASTType.Function:
-            term = Function(term.location, self._rename_function(term.name), term.arguments, term.external)
-        return SymbolicAtom(term)
+        new_term = self._transformer_function(term)
+        return x if new_term is term else SymbolicAtom(new_term)
+
+
+def rewrite_symbolic_atoms(x: AST, rewrite_function: Callable[[AST], AST]) -> AST:
+    '''
+    Rewrite all symbolic atoms in the given AST node with the given function.
+
+    Parameters
+    ----------
+    x
+        The ast in which to rename symbolic atoms.
+    rename_function
+        A function applied to the term representation of the symbolic atom. The
+        function has to return a term compatible with symbolic atoms.
+
+    Returns
+    -------
+    The rewritten AST.
+    '''
+    return cast(AST, _SymbolicAtomTransformer(rewrite_function)(x))
+
 
 def rename_symbolic_atoms(x: AST, rename_function: Callable[[str], str]) -> AST:
     '''
@@ -696,7 +714,19 @@ def rename_symbolic_atoms(x: AST, rename_function: Callable[[str], str]) -> AST:
     -------
     The rewritten AST.
     '''
-    return cast(AST, SymbolicAtomRenamer(rename_function)(x))
+    def renamer(term: AST):
+        if term.ast_type == ASTType.UnaryOperation:
+            return UnaryOperation(term.location, term.operator_type, renamer(term.argument))
+        if term.ast_type == ASTType.SymbolicTerm:
+            sym = term.symbol
+            new_name = rename_function(sym.name)
+            return SymbolicTerm(term.location, clingo.Function(new_name, sym.arguments, sym.positive))
+        if term.ast_type == ASTType.Function:
+            return Function(term.location, rename_function(term.name), term.arguments, term.external)
+        return term
+
+    return rewrite_symbolic_atoms(x, renamer)
+
 
 def prefix_symbolic_atoms(x: AST, prefix: str) -> AST:
     '''
@@ -720,38 +750,85 @@ def prefix_symbolic_atoms(x: AST, prefix: str) -> AST:
     return rename_symbolic_atoms(x, lambda s: prefix + s)
 
 
+def reify_symbolic_atoms(x: AST, name: str,
+                         argument_extender: Callable[[AST], Sequence[AST]] = None,
+                         reify_strong_negation: bool = False) -> AST:
+    '''
+    Reify all symbolic atoms in the given AST node with the given name and
+    function.
+
+    Parameters
+    ----------
+    x
+        The ast in which to rename symbolic atoms.
+    name
+        A string to serve as name of the new symbolic atom.
+    argument_extender
+        A function to provide extra arguments. If not provided, no extra
+        arguments are added. The term passed as argument should be placed in
+        the correct position.
+    reify_strong_negation
+        Boolean indicating how to encode strong negation. If false, `-p(X)` is
+        reified as `-name(p(X))`. If true, then -p(X) is reified as
+        name(-p(X)). In the latter case, this means that stable models
+        containing both `name(p(a))` and `name(-p(a))` are possible. Clingo
+        style consistency can be restored by adding the constraint
+        `:- name(X), name(-X), X<-X.`
+
+    Returns
+    -------
+    The rewritten AST.
+    '''
+
+    def reifier(term: AST):
+        if term.ast_type == ASTType.UnaryOperation and not reify_strong_negation:
+            return UnaryOperation(term.location, term.operator_type, reifier(term.argument))
+        arguments = argument_extender(term) if argument_extender else [term]
+        return Function(term.location, name, arguments, False)
+
+    return rewrite_symbolic_atoms(x, reifier)
+
+
 @singledispatch
 def _encode(x: Any) -> Any:
     assert False, f"unknown value to encode: {x}"
 
-@_encode.register
+
+@_encode.register(str)
 def _encode_str(x: str) -> str:
     return x
 
-@_encode.register
+
+@_encode.register(clingo.Symbol)
 def _encode_symbol(x: clingo.Symbol) -> str:
     return str(x)
 
-@_encode.register
+
+@_encode.register(int)
 def _encode_int(x: int) -> int:
     return x
 
-@_encode.register
+
+@_encode.register(ASTSequence)
 def _encode_ast_seq(x: ASTSequence) -> List[Any]:
     return [_encode(y) for y in x]
 
-@_encode.register
+
+@_encode.register(StrSequence)
 def _encode_str_seq(x: StrSequence) -> List[Any]:
     return [_encode(y) for y in x]
 
-@_encode.register
+
+@_encode.register(type(None))
 def _encode_none(x: None) -> None:
     return x
 
-@_encode.register
+
+@_encode.register(AST)
 def _encode_ast(x: AST) -> Any:
     return ast_to_dict(x)
 
+
 def ast_to_dict(x: AST) -> dict:
     """
     Convert the given ast node into a dictionary representation whose elements
@@ -788,7 +865,8 @@ def ast_to_dict(x: AST) -> dict:
 def _decode(x: Any, key: str) -> Any:
     raise RuntimeError(f"unknown key/value to decode: {key}: {x}")
 
-@_decode.register
+
+@_decode.register(str)
 def _decode_str(x: str, key: str) -> Any:
     if key == "location":
         return str_to_location(x)
@@ -799,26 +877,31 @@ def _decode_str(x: str, key: str) -> Any:
     assert key in ("name", "id", "code", "elements", "term", "list", "operator_name")
     return x
 
-@_decode.register
+
+@_decode.register(int)
 def _decode_int(x: int, key: str) -> Any:
     # pylint: disable=unused-argument
     return x
 
-@_decode.register
+
+@_decode.register(type(None))
 def _decode_none(x: None, key: str) -> Any:
     # pylint: disable=unused-argument
     return x
 
-@_decode.register
+
+@_decode.register(list)
 def _decode_list(x: list, key_: str) -> Any:
     # pylint: disable=unused-argument
     return [_decode(y, "list") for y in x]
 
-@_decode.register
+
+@_decode.register(dict)
 def _decode_dict(x: dict, key_: str) -> Any:
     # pylint: disable=unused-argument
     return dict_to_ast(x)
 
+
 def dict_to_ast(x: dict) -> AST:
     """
     Convert the Python dict representation of an AST node into an AST node.
